"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Directive = exports.CustomAttribute = exports.Modifier = exports.Input = void 0;
const helpers_1 = require("../helpers");
/**
 * Decorator @Input
 * Used to get attribute from element
 */
const Input = (options) => (target, memberName) => {
    const OnInit = target.OnInit || helpers_1.noop;
    Object.defineProperty(target, 'OnInit', {
        value() {
            var _a;
            let originalValue = this[memberName];
            const element = (_a = this.element) !== null && _a !== void 0 ? _a : this;
            Object.defineProperty(this, memberName, {
                get: function () {
                    originalValue = element.getAttribute(memberName.toLowerCase());
                    return originalValue;
                },
                set(value) {
                    element.setAttribute(memberName.toLowerCase(), value);
                    originalValue = value;
                },
                configurable: true,
            });
            return OnInit.call(this);
        },
        configurable: true,
    });
    if (options === null || options === void 0 ? void 0 : options.observe) {
        (0, helpers_1.observe)(target, memberName);
    }
};
exports.Input = Input;
/**
 * Decorator @Modifier
 * Accepts parameter options with selector and registry
 */
const Modifier = (options) => {
    return (target) => {
        target['options'] = options;
    };
};
exports.Modifier = Modifier;
/**
 * Decorator @CustomAttribute
 * Accepts parameter options with selector and registry
 */
exports.CustomAttribute = exports.Modifier;
/**
 * Decorator @Directive
 * Accepts parameter options with selector and registry
 */
exports.Directive = exports.Modifier;
//# sourceMappingURL=index.js.map