"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MediaQueryAttribute = exports.createFiltersFromSelector = exports.Breakpoints = exports.MediaMatchers = void 0;
const attribute_1 = require("./attribute");
/**
 * Media query Attribute
 * for performance reasons it is key value pair
 */
exports.MediaMatchers = new Map([
    ['screen and (max-width: 599px)', 'xs'],
    ['screen and (min-width: 600px) and (max-width: 959px)', 'sm'],
    ['screen and (min-width: 960px) and (max-width: 1279px)', 'md'],
    ['screen and (min-width: 1280px) and (max-width: 1919px)', 'lg'],
    ['screen and (min-width: 1920px) and (max-width: 5000px)', 'xl'],
    ['screen and (max-width: 959px)', 'lt-md'],
    ['screen and (max-width: 1279px)', 'lt-lg'],
    ['screen and (max-width: 1919px)', 'lt-xl'],
    ['screen and (min-width: 600px)', 'gt-xs'],
    ['screen and (min-width: 960px)', 'gt-sm'],
    ['screen and (min-width: 1280px)', 'gt-md'],
    ['screen and (min-width: 1920px)', 'gt-lg'],
]);
exports.Breakpoints = [...exports.MediaMatchers.values()];
const createFiltersFromSelector = (selector) => [
    ...exports.Breakpoints.map((breakpoint) => `${selector}.${breakpoint}`),
    selector,
];
exports.createFiltersFromSelector = createFiltersFromSelector;
class MediaQueryAttribute extends attribute_1.Attribute {
    constructor() {
        super(...arguments);
        this.matchers = new Map();
        this.cachedAttributes = new Map();
        this.listener = (event) => {
            const key = `${this.selector.toLowerCase()}.${exports.MediaMatchers.get(event.media)}`;
            const attribute = this.cachedAttributes.get(key);
            if (event.matches && attribute) {
                return this.OnEnterMediaQuery([event, attribute]);
            }
            return this.OnExitMediaQuery([event, attribute]);
        };
    }
    OnInit() {
        if (this.OnEnterMediaQuery || this.OnExitMediaQuery) {
            for (const query of exports.MediaMatchers.keys()) {
                const matcher = window.matchMedia(query);
                const attr = Object.values(this.element.attributes).find((v) => v.name ===
                    `${this.selector.toLowerCase()}.${exports.MediaMatchers.get(query)}`);
                if (attr) {
                    this.cachedAttributes.set(attr.name, attr);
                    matcher.addEventListener('change', this.listener);
                }
                if (attr && matcher.matches) {
                    this.listener(matcher);
                }
                this.matchers.set(matcher, matcher);
            }
        }
    }
    OnDestroy() {
        for (const matcher of this.matchers.values()) {
            matcher.removeEventListener('change', this.listener);
        }
        this.cachedAttributes.clear();
        this.matchers.clear();
    }
}
exports.MediaQueryAttribute = MediaQueryAttribute;
//# sourceMappingURL=media-attribute.js.map